from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.CMFCore.CMFCorePermissions import View, ManageProperties, ListFolderContents
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.CMFCorePermissions import *

from Products.CMFDefault.File import File
from global_symbols import *
from Products.CMFCore.utils import getToolByName

import os

from Products.PloneiCalendar.parser import *
import datetime
import string
from StringIO import StringIO

# WebDAV support (in order to refresh the calendar views after a WebDAV upload)
from webdav.Lockable import ResourceLockedError
from webdav.WriteLockInterface import WriteLockInterface

factory_type_information = {
  'id'             : 'iCalendar',
  'portal_type'    : 'iCalendar',
  'meta_type'      : 'iCalendar',
  'description'    : 'An PloneiCalendar is a content type with an ics file attached.',
  'content_icon'   : 'PloneiCalendar.gif',
  'product'        : 'PloneiCalendar',
  'factory'        : 'addPloneiCalendar',
  'immediate_view' : 'PloneiCalendar_editForm',
  'filter_content_types' : 0,
  'allowed_content_types': (),
  'actions': (
    {
    'id'           : 'view',
    'name'         : 'View',
    'action'       : 'string:${object_url}/PloneiCalendar_view_by_day',
    'permissions'  : (CMFCorePermissions.View,),
    'category'     : 'object',
    },
    {
    'id'           : 'links',
    'name'         : 'Links',
    'action'       : 'string:${object_url}/PloneiCalendar_view',
    'permissions'  : (CMFCorePermissions.View,),
    'category'     : 'object',
    },
    {
    'id'            : 'edit',
    'name'          : 'Edit',
    'action'        : 'string:${object_url}/PloneiCalendar_editForm',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'            : 'local_roles',
    'name'          : 'Sharing',
    'action'        : 'string:${object_url}/PloneiCalendar_localroleForm',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'condition'     : '',
    'permission'    : 'Manage properties',
    'category'      : 'object'
    },
    {
    'id'            : 'metadata',
    'name'          : 'Properties',
    'action'        : 'string:${object_url}/metadata_edit_form',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'            : 'content_status_history',
    'name'          : 'State',
    'action'        : 'string:${object_url}/content_status_history',
    'condition'     : 'python:object and portal.portal_workflow.getTransitionsFor(object, object.getParentNode())',
    'permissions'   : (CMFCorePermissions.View,),
    'category'      : 'object_tabs',
    'visible'       : 0
    },
   
    ),
  }
  

# The usual factory
def addPloneiCalendar(self, id, file='', REQUEST = {}):
    """
    Factory method for a PloneiCalendar object
    """
    obj = PloneiCalendar(id, file )
    self._setObject(id, obj)
    getattr(self, id)._post_init()
    
    if REQUEST.has_key('RESPONSE'):
        return REQUEST.RESPONSE.redirect('manage_main')


class PloneiCalendar(File, ):
    __implements__ = ( File.__implements__, WriteLockInterface, )
    meta_type = factory_type_information['meta_type']

    # Standard security settings
    security = ClassSecurityInfo()
    
    webcolor = '' # Color of events in this calendar
    
    security.declarePrivate('getEmptyFile')
    def getEmptyFile(self,):
        """ Return an empty ics raw content """
        emptyFileName = "vierge.ics"
        j = os.path.join
        var_dir = j(j(INSTANCE_HOME, 'Products'), 'PloneiCalendar')
        file = ''
        try:
            file = open(j(var_dir, emptyFileName))
        finally:
            return file

    # The following method makes all class properties & methods protected by View by default
    #security.declareObjectProtected(CMFCorePermissions.View)
    
    def __init__(self, id, file=''):
        """ Init method """
        File.__init__( self, id, file=file )
        if not file:
            file = self.getEmptyFile()
        self.manage_upload( file=file )
    
    index_html = None  # This special value informs ZPublisher to use __call__

    security.declarePrivate('_post_init')
    def _post_init(self):
        """
        _post_init(self) => Post-init method (that is, method that is called AFTER the class has been set into the ZODB)
        """
        # We set the object creator as a Reviewer - as it won't be possible to do so later because of permission restrictions
        obj = self
        portal_membership = getToolByName(self, 'portal_membership')
        member_id = portal_membership.getAuthenticatedMember().getUserName()
        roles = list(obj.get_local_roles_for_userid( userid=member_id ))
        if "LocalWriter" not in roles and "LocalReader" not in roles :
            roles.append( "LocalWriter" )
            roles.append( "LocalReader" )
            obj.manage_setLocalRoles( member_id, roles )
        Log(LOG_DEBUG, "local roles", self.get_local_roles_for_userid(member_id))
        # We reindex the object
        self.indexObject()
        ### New in version 1.0
        self._v_tree=None # the volatile list of VEvents


    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'manage_editPloneiCalendar')
    def manage_editPloneiCalendar(self, title = '', file=None, id=id, description='', REQUEST = {}, webcolor = ''):
        """
        #edit(self, title = '', REQUEST = {}) => object modification method
        """
        if self.wl_isLocked():
          raise ResourceLockedError, "This calendar is locked by WebDAV"
        # Change attributes
        self.id = id
        self.title = title
        self.description = description
        self.webcolor = webcolor
        if file:
            self.manage_upload( file=file )
        # Reindex
        self.reindexObject()
    
    security.declareProtected(CMFCorePermissions.View, "getICSUrl")
    def getICSUrl(self,):
        """ Return an url to access the ics file """
        return self.ics_url()
        #url = self.absolute_url()
        #return replace(url, 'http', 'webcal')
    
    security.declareProtected(CMFCorePermissions.View, "returnFileContent")
    def returnFileContent(self,):
        """ Return this PloneiCalendar's raw content """
        if self.data:
            return self.data
        return 'no file data'
    
    
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, "upload")
    def upload(self, file):
        """ Given a file name, upload this file to this PloneiCalendar """
        if file:
            self.manage_upload(file)
            if hasattr(self,"_v_tree"):
              del(self._v_tree)

    ### New in version 1.0
    
    #_v_tree : [VEvent]
    #_v_rd : dictionnary {uid:VEvent}
    
    security.declarePrivate('_parse')
    def _parse(self):
      """ Parses the attached ICS file and fills _v_tree with the corresponding
      events (VEvent objects). Raises an exception if the parser fails. """
      datas = getToolByName(self, 'portal_ploneicalendar').convertStringToPortalCharset(self, self.data)
      icr = ICalReader(StringIO(datas))
      lvevent = list(icr.iterEvents())
      lvevent.sort()
      dvevent = dict([(ev.getUID(),ev) for ev in lvevent if ev.getUID()])
      self._v_tree=lvevent
      self._v_rd=dvevent
      pass
    
    security.declarePrivate('accessVolatile')
    def accessVolatile(self):
      """ Verify that the volatile datas are still here before accessing it. If not, parse again the ics file """
      if not (hasattr(self,"_v_tree") and hasattr(self,"_v_rd")):
        self._parse()

    security.declareProtected(CMFCorePermissions.View, "listEvents")
    def listEvents(self,date1=datetime.datetime.today().replace(hour=8,minute=0,second=0), date2=datetime.datetime.today().replace(hour=22,minute=0,second=0), onlyFullEvents=0):
      """ Updates _v_tree if necessary, and then returns a sorted list
      (chronological order) with this object's events (VEvent objects) that
      occur between date1 and date2, including recurrent ones. """
      self.accessVolatile()
      lvev=[]
      for vev in self._v_tree:
        lvev=lvev+vev.getOccurrencesBetween(date1,date2,onlyFullEvents)
      lvev.sort()
      return lvev

    security.declareProtected(CMFCorePermissions.View, "getVEventByUID")
    def getVEventByUID(self, uid):
      """ Return the VEvent with the corresponding uid or None if it doesn't exist """
      self.accessVolatile()
      return self._v_rd.get(uid,None)
      
    # WebDAV support
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, "PUT")
    def PUT(self, REQUEST, RESPONSE):
        """ Handle HTTP PUT requests """
        self.dav__init(REQUEST, RESPONSE)
        self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
        # update the calendar content
        content=REQUEST.get('BODY', '')
        self.update_data(content, content_type='text/calendar', size=len(content))
        RESPONSE.setStatus(204)
        # Refresh the calendar object
        self._parse()
        return RESPONSE

    security.declareProtected(CMFCorePermissions.View, "manage_FTPget")
    def manage_FTPget(self):
        """ Get source for FTP download """
        self.REQUEST.RESPONSE.setHeader('Content-Type', 'text/calendar')
        # return the raw calendar content
        return self.returnFileContent()

    # Functions that should definitively be moved into a tool or a set of PythonScripts
    # accompanying the product.
    
    security.declareProtected(CMFCorePermissions.View, "listAllCalendars")
    def listAllCalendars(self):
      """ Return a list of all PloneiCalendar objects in this portal. """
      return getToolByName(self, 'portal_ploneicalendar').listAllCalendars(self)
      
    security.declareProtected(CMFCorePermissions.View, "listCalendars")
    def listCalendars(self, user=None, readable=0, writeable=0):
      """ Return a list of all PloneiCalendar objects in this portal readable
      or writeable by the user (as specified). If a parameter equals 0, it is
      ignored. If the user is not given, assume that it is the current one. """
      return getToolByName(self, 'portal_ploneicalendar').listCalendars(self, user, readable, writeable)

InitializeClass(PloneiCalendar)
