from Products.CMFCore.utils import UniqueObject 
from OFS.SimpleItem import SimpleItem 
from Globals import InitializeClass 
from Products.CMFCore.utils import getToolByName
from AccessControl import ClassSecurityInfo
from OFS.PropertyManager import PropertyManager
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.CMFCorePermissions import View, ManageProperties, ListFolderContents

import time, types, string
try :
	import datetime
except:
        raise ImportError('Module datetime not found, make sure you have Python 2.3 or greater.')
		

class PloneiCalendarTool(UniqueObject, SimpleItem, PropertyManager):
  
  """ PloneiCalendar Tool """ 
  
  id = 'portal_ploneicalendar' 
  meta_type= 'PloneiCalendar Tool' 
  plone_tool = 1 
  
  security = ClassSecurityInfo()
  
  manage_options = SimpleItem.manage_options + PropertyManager.manage_options
  

  def dt2ts(self, dt):
	""" Convert this datetime to an epoch timestamp """
	return int(time.mktime(dt.timetuple()))

  def ts2dt(self, ts):
	""" Convert this epoch timestamp to a datetime """
	return datetime.datetime.fromtimestamp(ts)	

  security.declareProtected(CMFCorePermissions.View, "getTodayTs")
  def getTodayTs(self):
    """ Return today's timestamp """
    return self.dt2ts(datetime.datetime.now().replace(hour=0,minute=0,second=0))
  
  ### Date accessors ###
  
  security.declareProtected(CMFCorePermissions.View, "getTs")
  def getTs(self, context):
    """ Return the timestamp from context or today's timestamp if it is not found """
    return int(context.REQUEST.get('ts',self.getTodayTs()))
  
  security.declareProtected(CMFCorePermissions.View, "getDatetime")
  def getDatetime(self, context):
    """ Return the corresponding datetime object from context """
    return self.getDatetimeFromTs(self.getTs(context))
  
  security.declareProtected(CMFCorePermissions.View, "getWeekday")
  def getWeekday(self, dt):
    """ Return the weekday number from a datetime """
    wd=int(dt.strftime('%w'))
    # In order to start the week with Monday [0..6] : to be customized ?
    wd=(wd+6)%7
    return wd
    
  security.declareProtected(CMFCorePermissions.View, "getWeekNumber")
  def getWeekNumber(self, dt):
    """ Return the week number in this datetime year """
    wn=int(dt.strftime('%W'))+1
    #Should be %U for english people ?
    return wn
  
  security.declareProtected(CMFCorePermissions.View, "getFirstDayOfWeek")
  def getFirstDayOfWeek(self, dt):
    """ Return a datetime of the first day of week from this datetime """
    dt2=dt
    while(self.getWeekday(dt2)):
      dt2=dt2-datetime.timedelta(days=1)
    return dt2
  
  security.declareProtected(CMFCorePermissions.View, "getFirstDayOfMonth")
  def getFirstDayOfMonth(self, dt):
    """ Return a datetime of the first day of month from this datetime """
    return dt.replace(day=1)
  
  security.declareProtected(CMFCorePermissions.View, "getFirstDayOfMonthTs")
  def getFirstDayOfMonthTs(self, dt):
    """ Return a timestamp of the first day of month from this datetime """
    return self.dt2ts(self.getFirstDayOfMonth(dt))
  
  security.declareProtected(CMFCorePermissions.View, "getWeekdayName")
  def getWeekdayName(self, dt):
    """ Return the weekday's i18n id from this datetime"""
    return "calendar_weekday_"+str(self.getWeekday(dt))

  security.declareProtected(CMFCorePermissions.View, "getMonthName")
  def getMonthName(self, dt):
    """ Return the monthname's i18n id from this datetime"""
    return "calendar_month_"+str(dt.month)
  
  security.declareProtected(CMFCorePermissions.View, "addMonths")
  def addMonths(self, ts, n):
    """ Add a number of months (postive or negative) to this timestamp and return a timestamp"""
    dt=self.getDatetimeFromTs(ts)
    d,m,y=dt.day,dt.month+n,dt.year
    while m<1:
      m=m+12
      y=y-1
    while m>12:
      m=m-12
      y=y+1
    while True:
      try:
        dt=dt.replace(day=d,month=m,year=y)
        break
      except ValueError:
        d=d-1
    return self.dt2ts(dt)
    
  ### Converters ###

  security.declareProtected(CMFCorePermissions.View, "getDatetimeFromTs")
  def getDatetimeFromTs(self, ts):
    """ Return the corresponding datetime from this timestamp """
    return datetime.datetime.fromtimestamp(ts)

  security.declareProtected(CMFCorePermissions.View, "timedelta2seconds")
  def timedelta2seconds(self, td):
    """ Return the equivalent number of seconds of this timedelta """
    return td.seconds+td.days*60*60*24
  
  security.declareProtected(CMFCorePermissions.View, "cutstring")
  def cutstring(self, str1, length):
    """ Return a shorter version of the string str1 by keeping only the first
    (length-3) characters and adding "..." if it is longer than length characters. """
    if len(str1)<length:
      return str1
    else:
      return str1[:(length-3)]+"..."

  security.declareProtected(CMFCorePermissions.View, "convertStringToPortalCharset")
  def convertStringToPortalCharset(self, context, string1):
    """ Return the given string converted to the current portal charset """
    site_charset = context.portal_properties.site_properties.default_charset
    string2 = None
    try:
      string2 = string1.decode('utf-8')
    except UnicodeError:
      try :
        string2 = string1.decode('iso-8859-1','ignore')
      except UnicodeError:
        string2 = string1
    return string2.encode(site_charset)
  
  security.declareProtected(CMFCorePermissions.View, "blackify")
  def blackify(self, color, percent=40) :
    """ Remove 'percent' of the given color (html string format) """
    if not color:
	color="#ffffff"
    percent=int(percent)

    def black(x):
      i=int(x,16)                       # convert to integer
      i=int( i*(1.0-(percent/100.0)) )  # remove percentage
      x=hex(i)[2:]                      # back to hex
      if len(x)==1:                   
        x='0'+x                         # leading 0 if necessary
      return x

    if color[0]=='#': # remove leading '#' if present
      color=color[1:]

    colors=(color[0:2],color[2:4],color[4:6]) # slice colors
    colors=map(black,colors)  # blackify

    return '#'+string.join(colors,'') # rebuild HTML color

  ### Date formatters ###
  
  security.declareProtected(CMFCorePermissions.View, "getFormattedDateFromDt")
  def getFormattedDateFromDt(self, context, dt, format) :
    """ Given a translation context, a date or datetime and an i18n msgid, return the translated date """
    if dt:
      return context.translate(
          domain="ploneicalendar",
          msgid=format,
          mapping={
            'weeknumber':   str(self.getWeekNumber(dt)),
            'weekday':      str(self.getWeekday(dt)),
            'weekdayname':  context.translate(domain="ploneicalendar",msgid=self.getWeekdayName(dt)),
            'day':          str(dt.day),
            'month':        str(dt.month),
            'monthname':    context.translate(domain="ploneicalendar",msgid=self.getMonthName(dt)),
            'year':         str(dt.year),
            'hour':         dt.strftime('%H'),
            'minute':       dt.strftime('%M'),
            'second':       dt.strftime('%S')
          }
      )
    else:
      return None
  
  security.declareProtected(CMFCorePermissions.View, "getFormattedDate")
  def getFormattedDate(self, context, format):
    """ Given a translation context, and an i18n msgid, return the translated context's date """
    dt=self.getDatetime(context)
    return self.getFormattedDateFromDt(context, dt, format)
  
  security.declareProtected(CMFCorePermissions.View, "getWeekDaysName")
  def getWeekDaysName(self, context, format):
    """ Return the formated weekdays columns needed by the week and month views """
    date=self.getDatetime(context)
    wdn=[]
    date=self.getFirstDayOfWeek(date)
    for i in range(7):
      wdn.append([13*i+9,self.getFormattedDateFromDt(context, date, format),str(self.dt2ts(date))])
      date=date+datetime.timedelta(days=1)
    return wdn
    
  ### Renderers ###

  security.declareProtected(CMFCorePermissions.View, "dayview")
  def dayview(self, context):
    """ Return a formatted list of all-day events and normal events occuring during this day """
    hourheight=30
    date=self.getDatetime(context)
    startdate=date.replace(hour=0,minute=0,second=0)
    enddate=startdate.replace(hour=23,minute=59,second=59)

    leven=context.listEvents(startdate,enddate)
    levendraw=[[],[]] #List of all day events and normal events : [summary,start,end,uid]
    c=0
    lkevents=[]
    for ev in leven:
      if ev[2].all_day_event:
        #All day events
        bodytext=[ev[2].summary,None,None]
        if ev[2].getUID():
          bodytext.append(ev[2].getUID())
        else:
          bodytext.append(None)
        bodytext.append(context.webcolor)
        levendraw[0].append(bodytext)
      else:
        #Normal events
        deltaposition=0
        for ke in lkevents:
          if ev[0]>=ke[0] and ev[0]<ke[1]:
            deltaposition=deltaposition+1
        bodytext=[ev[2].summary,ev[2].dtstart.strftime("%H:%M"),ev[2].dtend.strftime("%H:%M")]
        if ev[2].getUID():
          bodytext.append(ev[2].getUID())
        else:
          bodytext.append(None)
        boxheight=max(self.timedelta2seconds(ev[1]-ev[0])*hourheight/3600-1,hourheight-1)
        levendraw[1].append((
          deltaposition*5,
          self.timedelta2seconds(ev[0]-startdate)*hourheight/3600-1,
          boxheight,
          context.webcolor,
          self.blackify(context.webcolor),
          '#ffffff',
          bodytext))
        c=c+1
        lkevents.append(ev)
    return levendraw

  security.declareProtected(CMFCorePermissions.View, "weekview")
  def weekview(self, context):
    """ Return a formatted list of all-day events and normal events occuring during this week """
    hourheight=30
    date=self.getFirstDayOfWeek(self.getDatetime(context))
    startdate=date.replace(hour=0,minute=0,second=0)
    levendraw=[[[] for i in range(7)],[]]
    
    enddate=startdate.replace(hour=23,minute=59,second=59)
    for day in range(7):
      c=0
      lkevents=[]
      leven=context.listEvents(startdate,enddate)
      for ev in leven:
        if not ev[2].all_day_event:
          deltaposition=0
          for ke in lkevents:
            if ev[0]>=ke[0] and ev[0]<ke[1]:
              deltaposition=deltaposition+1
          bodytext=[ev[2].summary,ev[2].dtstart.strftime("%H:%M"),ev[2].dtend.strftime("%H:%M")]
          if ev[2].getUID():
            bodytext.append(ev[2].getUID())
          else:
            bodytext.append(None)
          boxheight=max(self.timedelta2seconds(ev[1]-ev[0])*hourheight/3600-1,hourheight-1)
          levendraw[1].append((
            day*13,
            self.timedelta2seconds(ev[0]-startdate)*hourheight/3600-1+2*hourheight+deltaposition*15,
            boxheight,
            context.webcolor,
            self.blackify(context.webcolor),
            '#ffffff',
            bodytext))
          c=c+1
          lkevents.append(ev)
        else:
          #All day events
          bodytext = [ev[2].summary,context.webcolor]
          if ev[2].getUID():
            bodytext.append(ev[2].getUID())
          else:
            bodytext.append(None)
          levendraw[0][day].append(bodytext)
      startdate=startdate+datetime.timedelta(days=1)
      enddate=enddate+datetime.timedelta(days=1)
      
    return levendraw
  
  security.declareProtected(CMFCorePermissions.View, "monthview") 
  def monthview(self, context):
    """ Return a formatted list of all-day events and normal events occuring during this month """
    dt=self.getDatetime(context).replace(day=1, hour=0, minute=0, second=0)
    dt2=dt
    wd=self.getWeekday(dt)
    dl=[]
    x,y=wd,0
    cellwidth=13 #in percent
    cellheight=10
    while dt2.month==dt.month:
      lvevent=map(lambda x: x[2],context.listEvents(dt2,dt2.replace(hour=23,minute=59,second=59)))
      lvevent.sort(lambda x,y: cmp((not x.all_day_event,x.summary),(not y.all_day_event,y.summary)))
      lvevtoday=[] # [summary, short summary, CSS class, href, color]
      evcounter=0
      for ev in lvevent:
        evcounter=evcounter+1
        if ev.all_day_event:
          lvevtoday.append([ev.summary,self.cutstring(ev.summary,12),"ploneicalendar_monthalldayevent","/PloneiCalendar_view_event?uid="+ev.getUID(),context.webcolor])
        else:
          lvevtoday.append(['['+ev.dtstart.strftime('%H:%M')+'-'+ev.dtend.strftime('%H:%M')+'] '+ev.summary,self.cutstring(ev.summary,12),"ploneicalendar_monthsimpleevent","/PloneiCalendar_view_event?uid="+ev.getUID(), context.webcolor])
      dl.append([x*cellwidth+9,y*cellheight,dt2.day,str(self.dt2ts(dt2)),lvevtoday])
      dt2=dt2+datetime.timedelta(days=1)
      x=x+1
      if x>6:
        x=x-7
        y=y+1
    return dl

  ### Other ###
  
  security.declareProtected(CMFCorePermissions.View, "stripHTML")
  def stripHTML(self, str1):
    """ Remove html tags in this string """
    if str1:
      return str1.replace('<','&lt;').replace('>','&gt;')
    else:
      return ""
  
  security.declareProtected(CMFCorePermissions.View, "getContent")
  def getContent(self,context,uid):
    """ Format the contents of the VEvent with the corresponding uid and return it """
    known_content={}
    unknown_content={}
    if uid:
      vev=context.getVEventByUID(uid)
      known_content=dict([(k,vev.getOne(k)) for k in ['SUMMARY','DESCRIPTION','LOCATION','URL','CATEGORIES']])
      known_content['DESCRIPTION']=self.stripHTML(known_content['DESCRIPTION'])
      known_content['SUMMARY']=self.stripHTML(known_content['SUMMARY'])
      known_content.update(dict([(k,self.getFormattedDateFromDt(context,vev.getOne(k),"long_datetime")) for k in ['DTSTART','DTEND','DTSTAMP','LAST-MODIFIED']]))
      if vev.dtstart.strftime("%d")==vev.dtend.strftime("%d"):
        #If the event fits in a single day
        known_content['date']=self.getFormattedDateFromDt(context,vev.getOne('DTSTART'),"long_date")
        if vev.all_day_event:
          known_content['start']=known_content['end']=None
        else:
          known_content['start']=self.getFormattedDateFromDt(context,vev.getOne('DTSTART'),"time")
          known_content['end']=self.getFormattedDateFromDt(context,vev.getOne('DTEND'),"time")
      else:
        #If the events fits in several days
        known_content['date']=None
        known_content['start']=self.getFormattedDateFromDt(context,vev.getOne('DTSTART'),"long_datetime")
        known_content['end']=self.getFormattedDateFromDt(context,vev.getOne('DTEND'),"long_datetime")
      for k in vev._props:
        if k not in known_content:
          unknown_content[k]=vev.getOne(k)
    return (known_content,unknown_content)
    
  ### Utility functions

  security.declareProtected(CMFCorePermissions.View, "listAllCalendars")
  def listAllCalendars(self, context):
    """ Return a list of all PloneiCalendar objects in this portal. """
    return [x.getObject() for x in context.portal_catalog(meta_type='iCalendar')]

  security.declareProtected(CMFCorePermissions.View, "listCalendars")
  def listCalendars(self, context, user=None, readable=0, writeable=0):
    """ Return a list of all PloneiCalendar objects in this portal readable
    or writeable by the user (as specified). If a parameter equals 0, it is
    ignored. If the user is not given, assume that it is the current one. """
    portal_membership = getToolByName(context, 'portal_membership')
    if not user:
      user=portal_membership.getAuthenticatedMember()
    else:
      user=portal_membership.getMemberById(user) or portal_membership.getAuthenticatedMember()
    lac=self.listAllCalendars(context)
    lr=[]
    for cal in lac:
      if (readable and not writeable and user.has_role("LocalReader", cal)) or (writeable and user.has_role("LocalWriter", cal)) or (not readable and not writeable):
          lr.append(cal)
    lr.sort(lambda x,y: cmp(x.title_or_id(),y.title_or_id()))
    return lr

InitializeClass(PloneiCalendarTool)
