#
# PloneiCalendar tests
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))


from Products.PloneTestCase import PloneTestCase
PloneTestCase.installProduct('PloneiCalendar')

PloneTestCase.setupPloneSite()

from Products.PloneiCalendar.global_symbols import *
from Acquisition import aq_base
from datetime import datetime
from Products.CMFCore.utils import getToolByName

try:
    import Log

    Log.LOG_LEVEL = Log.LOG_DEBUG

    Log.LOG_PROCESSOR = {
        Log.LOG_NONE: Log.logFile,
        Log.LOG_CRITICAL: Log.logFile,
        Log.LOG_ERROR: Log.logFile,
        Log.LOG_WARNING: Log.logFile,
        Log.LOG_NOTICE: Log.logFile,
        Log.LOG_DEBUG: Log.logFile,
        }
    from Log import *
    Log(LOG_NOTICE, "Starting %s at %d debug level" % (os.path.dirname(__file__), LOG_LEVEL, ))
except:
    print "Log module not available"
    LOG_DEBUG = None
    LOG_NOTICE = None
    LOG_WARNING = None
    LOG_ERROR = None
    LOG_CRITICAL = None
    def Log(*args, **kw):
        pass
    raise


# Fake upload object
class TestFile:
    __allow_access_to_unprotected_subobjects__ = 1
    filename = 'agenda.ics'
    def seek(*args): pass
    def tell(*args): return 0
    def read(*args): return 'file_contents'



class TestPloneiCalendar(PloneTestCase.PloneTestCase):

    def installPloneiCalendar(self):
        self.qi = self.portal.portal_quickinstaller
        self.qi.installProduct('PloneiCalendar')
        
    def afterSetUp(self, ):
        self.installPloneiCalendar()
        
        self.catalog = self.portal.portal_catalog
        self.workflow = self.portal.portal_workflow
        self.membership = self.portal.portal_membership
        self.contentId = 'agenda'
        self.folder.invokeFactory(type_name='iCalendar', id=self.contentId)
        self.failUnless(self.catalog(id=self.contentId))
        self.ical = getattr(self.folder, self.contentId)
        
    
    def testAddPloneiCalendar(self):
        """ test if PloneiCalendar is created """
        self.failUnless(hasattr(aq_base(self.folder), self.contentId))

    def testIsIndexed(self):
        """ test if PloneiCalendar is indexed """
        #test if the icalendar is indexed
        self.failUnless(self.catalog(id=self.contentId))
        Log(LOG_NOTICE, "PloneiCalendar indexed.")

    def testReindexPloneiCalendar(self):
        """ Modify the title and re index PloneiCalendar """
        self.failIf(self.catalog(id=self.contentId, Title='My Agenda'))
        self.ical.setTitle('My Agenda')
        self.ical.reindexObject()
        self.failUnless(self.catalog(id=self.contentId, Title='My Agenda'))
        Log(LOG_NOTICE, "PloneiCalendar reindexed.")
    
    #Standard Plone tools
    def testRenamePloneiCalendar(self):
        Log(LOG_NOTICE, "PloneiCalendar renamed.")
    
    def testCopyPloneiCalendar(self,):
        Log(LOG_NOTICE, "PloneiCalendar copied.")
    
    def testPastePloneiCalendar(self,):
        Log(LOG_NOTICE, "PloneiCalendar copied.")
    
    def testDeletePloneiCalendar(self,):
        """ delete PloneiCalendar """
        self.folder._delObject(self.contentId)
        self.failIf(self.catalog(id=self.contentId))
        Log(LOG_NOTICE, "PloneiCalendar deleted.")
    
    ###############################
    #PloneiCalendar related method#
    ###############################
    
    def testEditPloneiCalendar(self,):
        """ edit the PloneiCalendar """
        title = 'My agenda'
        description = 'Contains my dating for 2004'
        self.ical.setTitle(title)
        self.ical.setDescription(description)
        #edit our product with title and description
        #self.ical.manage_editPloneiCalendar(title, description)
        
        self.assertEqual(self.ical.Title(), title)
        self.assertEqual(self.ical.Description(), description)
        Log(LOG_NOTICE, "PloneiCalendar edited.")
    
    
    def testUploadICSFile(self,):
        """ upload a file """
        self.ical.manage_upload(file=TestFile())
        self.assertEqual(str(self.ical), 'file_contents')
        Log(LOG_NOTICE, "iCalendar file uploaded.")
    
    def testPloneiCalendarTotal(self,):
        """ Total test of PloneiCalendar """
        title = 'my personal agenda'
        description = 'an example of agenda'
        self.ical.manage_editPloneiCalendar(
                                            title = title, 
                                            file=TestFile(), 
                                            id=self.contentId, 
                                            description=description
                                           )
        self.failUnless(self.catalog(title=title))
        self.failUnless(self.catalog(description=description))
        self.assertEqual(str(self.ical), 'file_contents')
        Log(LOG_NOTICE, "iCalendar file uploaded.")
        
    ### New in version 1.0
        
    def testRecurEvents(self):
        from Products.PloneiCalendar.parser import VEvent
        
        ##################
        # Events creation
        ##################
        event1 = VEvent()
        event1.add("SUMMARY","Test 1")
        event1.add("DTSTART","20040701T100000Z")
        event1.add("DTEND",  "20040701T110000Z")
        event1.validate()
        
        event2 = VEvent()
        event2.add("SUMMARY","Test 2")
        event2.add("DTSTART","20040701T100000Z")
        event2.add("DTEND",  "20040701T110000Z")
        event2.add("RRULE",  "FREQ=WEEKLY;COUNT=6")
        event2.validate()
        
        event3 = VEvent()
        event3.add("SUMMARY","Test 3")
        event3.add("DTSTART","20040701T100000Z")
        event3.add("DTEND",  "20040701T110000Z")
        event3.add("RRULE",  "FREQ=DAILY;INTERVAL=10")
        event3.validate()
        
        event4 = VEvent()
        event4.add("SUMMARY","Test 4")
        event4.add("DTSTART","20040531T230000Z")
        event4.add("DTEND",  "20040601T010000Z")
        event4.add("RRULE",  "FREQ=MONTHLY;UNTIL=20040815T120000Z")
        event4.validate()
        
        ### Event 1
        #before
        self.assertEqual(event1.getOccurrencesBetween(datetime(2004,6,30,23,0,0),datetime(2004,7,1,9,0,0)),[])
        #during
        self.assertEqual(event1.getOccurrencesBetween(datetime(2004,7,1,9,0,0),datetime(2004,7,1,12,0,0)),[(datetime(2004,7,1,10,0,0),datetime(2004,7,1,11,0,0),event1)])
        self.assertEqual(event1.getOccurrencesBetween(datetime(2004,7,1,9,0,0),datetime(2004,7,1,10,15,0)),[(datetime(2004,7,1,10,0,0),datetime(2004,7,1,10,15,0),event1)])
        #after
        self.assertEqual(event1.getOccurrencesBetween(datetime(2004,7,1,23,0,0),datetime(2004,7,2,9,0,0)),[])
        
        ### Event 2
        self.assertEqual(event2.getOccurrencesBetween(datetime(2004,7,1,9,0,0),datetime(2004,9,1,12,0,0)),
          [(datetime(2004,7,1,10,0,0),datetime(2004,7,1,11,0,0),event2),
          (datetime(2004,7,8,10,0,0),datetime(2004,7,8,11,0,0),event2),
          (datetime(2004,7,15,10,0,0),datetime(2004,7,15,11,0,0),event2),
          (datetime(2004,7,22,10,0,0),datetime(2004,7,22,11,0,0),event2),
          (datetime(2004,7,29,10,0,0),datetime(2004,7,29,11,0,0),event2),
          (datetime(2004,8,5,10,0,0),datetime(2004,8,5,11,0,0),event2)])
        
        ### Event 3
        self.assertEqual(event3.getOccurrencesBetween(datetime(2004,6,1,9,0,0),datetime(2004,8,25,12,0,0)),
          [(datetime(2004,7,1,10,0,0),datetime(2004,7,1,11,0,0),event3),
          (datetime(2004,7,11,10,0,0),datetime(2004,7,11,11,0,0),event3),
          (datetime(2004,7,21,10,0,0),datetime(2004,7,21,11,0,0),event3),
          (datetime(2004,7,31,10,0,0),datetime(2004,7,31,11,0,0),event3),
          (datetime(2004,8,10,10,0,0),datetime(2004,8,10,11,0,0),event3),
          (datetime(2004,8,20,10,0,0),datetime(2004,8,20,11,0,0),event3)])
          
        ### Event 4
        self.assertEqual(event4.getOccurrencesBetween(datetime(2004,6,1,9,0,0),datetime(2004,8,25,12,0,0)),
          [(datetime(2004,6,30,23,0,0),datetime(2004,7,1,1,0,0),event4),
          (datetime(2004,7,31,23,0,0),datetime(2004,8,1,1,0,0),event4)])
        
        Log(LOG_NOTICE, "PloneiCalendar: recurring events tests succeeded.")
        
    def testEncodingDecoding(self,):
      """ Test encoding and decoding from/to Unicode, ISO-8859-1, UTF-8 """
      tool = getToolByName(self.ical, 'portal_ploneicalendar')
      
      fp = open("datas/sampletext_unicode.txt")
      unicodetext = fp.read()
      fp.close()
      
      fp = open("datas/sampletext_iso8859-1.txt")
      iso88591text = fp.read()
      fp.close()
 
      fp = open("datas/sampletext_utf8.txt")
      utf8text = fp.read()
      fp.close()

      self.ical.portal_properties.site_properties.default_charset = 'utf-8'
      self.assertEqual(tool.convertStringToPortalCharset(self.ical, iso88591text), utf8text)
      self.assertEqual(tool.convertStringToPortalCharset(self.ical, utf8text), utf8text)
      
      self.ical.portal_properties.site_properties.default_charset = 'utf-16'
      self.assertEqual(tool.convertStringToPortalCharset(self.ical, iso88591text), unicodetext)
      self.assertEqual(tool.convertStringToPortalCharset(self.ical, utf8text), unicodetext)
      
      self.ical.portal_properties.site_properties.default_charset = 'iso-8859-1'
      self.assertEqual(tool.convertStringToPortalCharset(self.ical, iso88591text), iso88591text)
      self.assertEqual(tool.convertStringToPortalCharset(self.ical, utf8text), iso88591text)
      
      self.ical.portal_properties.site_properties.default_charset = 'iso-8859-15'
      self.assertEqual(tool.convertStringToPortalCharset(self.ical, iso88591text), iso88591text)
      self.assertEqual(tool.convertStringToPortalCharset(self.ical, utf8text), iso88591text)
      
      Log(LOG_NOTICE, "PloneiCalendar: all encoding/decoding tests succeeded.")
    
if __name__ == '__main__':
    framework()
else:
    def test_suite():
        from unittest import TestSuite, makeSuite
        suite = TestSuite()
        suite.addTest(makeSuite(TestPloneiCalendar))
        return suite

