#
# PloneiCalendar tests
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.CMFPlone.tests import PloneTestCase
from Products.PloneiCalendar.global_symbols import *
from Acquisition import aq_base
from AccessControl.SecurityManagement import newSecurityManager, noSecurityManager

from AccessControl import getSecurityManager, Unauthorized


from Products.PloneTestCase import PloneTestCase
PloneTestCase.installProduct('PloneiCalendar')

PloneTestCase.setupPloneSite()



try:
    import Log

    Log.LOG_LEVEL = Log.LOG_DEBUG

    Log.LOG_PROCESSOR = {
        Log.LOG_NONE: Log.logFile,
        Log.LOG_CRITICAL: Log.logFile,
        Log.LOG_ERROR: Log.logFile,
        Log.LOG_WARNING: Log.logFile,
        Log.LOG_NOTICE: Log.logFile,
        Log.LOG_DEBUG: Log.logFile,
        }
    from Log import *
    Log(LOG_NOTICE, "Starting %s at %d debug level" % (os.path.dirname(__file__), LOG_LEVEL, ))
except:
    print "Log module not available"
    LOG_DEBUG = None
    LOG_NOTICE = None
    LOG_WARNING = None
    LOG_ERROR = None
    LOG_CRITICAL = None
    def Log(*args, **kw):
        pass
    raise


# Fake upload object
class TestFile:
    __allow_access_to_unprotected_subobjects__ = 1
    filename = 'agenda.ics'
    def seek(*args): pass
    def tell(*args): return 0
    def read(*args): return 'file_contents'
    

def sortTuple(t):
    l = list(t)
    l.sort()
    return tuple(l)

class TestPloneiCalendarSecurity(PloneTestCase.PloneTestCase):

    #utility method
        
    def createMember(self, id, pw, email, roles=('Member',)):
        pr = self.portal.portal_registration
        member = pr.addMember(id, pw, roles, properties={ 'username': id, 'email' : email })
        return member
    
    def createGroup(self, id):
        pg = self.portal.portal_groups
        pg.addGroup(id, '', (), ())
        return pg.getGroupById(id)
    
    def installPloneiCalendar(self):
        #self.loginAsPortalOwner()
        self.qi = self.portal.portal_quickinstaller
        self.qi.installProduct('PloneiCalendar')
    
    def setupSecurityContext(self,):
        self.logout()
        self.loginAsPortalOwner()
        #local Role
        self.localRoleWriter = 'LocalWriter'
        self.localRoleReader = 'LocalReader'
        
        #create Role
        self.portal._addRole(self.localRoleReader)
        self.portal._addRole(self.localRoleWriter)
        
        #create a bunch of users
        self.user1 = self.createMember('user1', 'abcd4', 'abc@domain.tld')
        self.user2 = self.createMember('user2', 'abcd4', 'abc@domain.tld')
        self.user3 = self.createMember('user3', 'abcd4', 'abc@domain.tld')
        
        self.failUnless('user1' in self.portal.Members.objectIds())
        self.failUnless('user2' in self.portal.Members.objectIds())
        self.failUnless('user3' in self.portal.Members.objectIds())
        
        #create some groups
        self.group1 = self.createGroup('group1')
        self.group2 = self.createGroup('group2')
        
        # Cannot assign a role I do not have myself...
        self.setGroups('group1')
        self.setRoles(['Member', self.localRoleReader, self.localRoleWriter])
    
        self.group1.addMember('user1')
        
        #just in case ...
        self.logout()
        self.login('user1')
        self.setRoles(['Member', self.localRoleReader, self.localRoleWriter], 'user1')
        self.setRoles(['Member'], 'user2')
        self.folder = self.portal.portal_membership.getHomeFolder('user1')
        
        
        
    
    
        

    def afterSetUp(self, ):
        #some usefull properties
        self.catalog = self.portal.portal_catalog
        self.workflow = self.portal.portal_workflow
        self.membership = self.portal.portal_membership
        
        #install product
        self.installPloneiCalendar()
        
        #setup users and groups
        self.setupSecurityContext()
        
        #create a instance of PloneiCalendar
        self.contentId = 'agenda'
        self.folder.invokeFactory(type_name='iCalendar', id=self.contentId)
        self.ical = self.folder.agenda
       
    #security
    def testGiveUserReadAccess(self,):
        """
        Give a user local role : Reader
        """
        verifRoles = sortTuple(('Authenticated', 'Member', self.localRoleReader))
        
        #user has not the local role
        self.failIf(self.user2.has_role(self.localRoleReader, self.ical))
        #Assign a role
        self.ical.folder_localrole_edit('add', ['user2'], self.localRoleReader)
        userLocalRoles = self.user2.getRolesInContext(self.ical)
        #test
        self.assertEqual(sortTuple(userLocalRoles), verifRoles)
        Log(LOG_NOTICE, "User2 has the local role " + self.localRoleReader)

    def testReaderAccess(self,):
        
        #user2 is not allowed to read
        self.login('user2')
        self.assertRaises(Unauthorized, self.ical.restrictedTraverse, "view")
        self.logout()
        
        #user1 grants user2 read access
        self.login('user1')
        self.testGiveUserReadAccess()
        self.logout()
        
        #user2 can view but not edit
        self.login('user2')
        self.assertEqual(self.membership.getAuthenticatedMember().getUserName(), 'user2')
        try:
            self.ical.restrictedTraverse("view")
        except Unauthorized:
            fail('Unauthorized')
        self.assertRaises(Unauthorized, self.ical.restrictedTraverse, "edit")
        

    def testGiveUserWriteAccess(self,):
        """
        Give a user local role : Writer
        """
        verifRoles = sortTuple(('Authenticated', 'Member', self.localRoleWriter))
        
        #user has not the local role
        self.failIf(self.user2.has_role(self.localRoleWriter, self.ical))
        #Assign a role
        self.ical.folder_localrole_edit('add', ['user2'], self.localRoleWriter)
        userLocalRoles = self.user2.getRolesInContext(self.ical)
        #test
        self.assertEqual(sortTuple(userLocalRoles), verifRoles)
        Log(LOG_NOTICE, "User2 has the local role " + self.localRoleWriter)
    

    def testRemoveUserWriteAccess(self,):
        """
        Give a user local role : Writer
        Remove the local role
        """
        # Should delete a local role
        
        
        self.ical.folder_localrole_edit('add', ['user2'], self.localRoleWriter)
        member = self.membership.getMemberById('user2')
        self.assertEqual(
                         sortTuple( member.getRolesInContext(self.ical) ),
                         sortTuple(( 'Authenticated', self.localRoleWriter, 'Member' ))
                        )
        
        self.ical.manage_delLocalRoles(['user2'])
        self.assertEqual(
                         sortTuple(member.getRolesInContext(self.ical)),
                         sortTuple(('Authenticated', 'Member'))
                        )
        
    def testRemoveUserReaderAccess(self,):
        """
        Give a user local role : Reader
        Remove the local role
        """
        
        #Assign a role
        self.ical.folder_localrole_edit('add', ['user2'], self.localRoleReader)
        member = self.membership.getMemberById('user2')
        self.assertEqual(
                         sortTuple( member.getRolesInContext(self.ical) ),
                         sortTuple(( 'Authenticated', self.localRoleReader, 'Member' ))
                        )
        
        self.ical.manage_delLocalRoles(['user2'])
        self.assertEqual(
                         sortTuple(member.getRolesInContext(self.ical)),
                         sortTuple(('Authenticated', 'Member'))
                        )

    def testGiveGroupReadAccess(self,):
        """
        Affect a local role (Reader) to a group 
        """
        verifRoles = sortTuple(('Authenticated', self.localRoleReader))
        
        #group doesn't have the local role
        self.failIf(self.group1.has_role(self.localRoleReader, self.ical))
        #Assign a role
        self.ical.folder_localrole_edit('add', ['group_group1'], self.localRoleReader)
        groupLocalRoles = sortTuple(self.group1.getRolesInContext(self.ical))
        #test
        self.assertEqual(groupLocalRoles, verifRoles)
        Log(LOG_NOTICE, "Group1 has the local role " + self.localRoleReader)
    
    def testGiveGroupWriteAccess(self,):
        """
        Affect a local role (Reader) to a group 
        """
        verifRoles = sortTuple(('Authenticated', self.localRoleWriter))
        
        #group doesn't have the local role
        self.failIf(self.group1.has_role(self.localRoleWriter, self.ical))
        #Assign a role
        self.ical.folder_localrole_edit('add', ['group_group1'], self.localRoleWriter)
        groupLocalRoles = sortTuple(self.group1.getRolesInContext(self.ical))
        #test
        self.assertEqual(groupLocalRoles, verifRoles)
        Log(LOG_NOTICE, "Group1 has the local role " + self.localRoleWriter)
    
    def testRemoveGroupWriterAccess(self,):
        """
        Give a group local role : Reader
        Remove the local role
        """
        #group doesn't have the local role
        self.failIf(self.group1.has_role(self.localRoleWriter, self.ical))
        #Assign a role
        self.ical.folder_localrole_edit('add', ['group_group1'], self.localRoleWriter)
        groupLocalRoles = sortTuple(self.group1.getRolesInContext(self.ical))
        #test
        self.assertEqual( groupLocalRoles, sortTuple(('Authenticated', self.localRoleWriter)) )
        
        self.ical.manage_delLocalRoles(['group_group1'])
        self.assertEqual(
                         self.group1.getRolesInContext(self.ical),
                         ('Authenticated', )
                        )
                        
    def testRemoveGroupReaderAccess(self,):
        """
        Give a group local role : Reader
        Remove the local role
        """
        #group doesn't have the local role
        self.failIf(self.group1.has_role(self.localRoleReader, self.ical))
        #Assign a role
        self.ical.folder_localrole_edit('add', ['group_group1'], self.localRoleReader)
        groupLocalRoles = sortTuple(self.group1.getRolesInContext(self.ical))
        #test
        self.assertEqual( groupLocalRoles, sortTuple(('Authenticated', self.localRoleReader)) )
        
        self.ical.manage_delLocalRoles(['group_group1'])
        self.assertEqual(
                         self.group1.getRolesInContext(self.ical),
                         ('Authenticated', )
                        )
    
    
        
        
    
    # basic accessibility 
    def testgetIcsUrlDenyAccess(self, ):
        """ ics url must no be seen by user2 """
        self.logout()
        self.login('user2')
        self.assertEqual(self.membership.getAuthenticatedMember().getUserName(), 'user2')
        self.assertRaises(Unauthorized, self.ical.restrictedTraverse, "getICSUrl")
    
    def testuploadDenyAccess(self, ):
        """ upload must not be used by another user """
        self.logout()
        self.login('user2')
        self.assertEqual(self.membership.getAuthenticatedMember().getUserName(), 'user2')
        self.assertRaises(Unauthorized, self.ical.restrictedTraverse, "upload")
        
    def testDenyAccessToAnonymous(self):
        """ Anonymous must not access, it is private by default """
        self.logout()
        self.assertRaises(Unauthorized, self.ical.restrictedTraverse, "view")

    ### New in version 1.0

    def testListAllCalendars(self): # Almost already done by testIsIndexed ?
        # Creates 3 PloneiCalendar objects
        self.folder.invokeFactory(type_name='iCalendar', id='ical1')
        self.folder.invokeFactory(type_name='iCalendar', id='ical2')
        self.folder.invokeFactory(type_name='iCalendar', id='ical3')
        # Checks that they are all listed by the listAllCalendars() function
        lobj = [x.getId() for x in self.ical.listAllCalendars()]
        lobj.sort()
        # We must find 4 elements : self.ical and ical1, ical2, ical3
        lobj2=[self.ical.getId(),'ical1','ical2','ical3']
        lobj2.sort()
        self.failUnless(lobj==lobj2)
        Log(LOG_NOTICE, 'PloneiCalendar: all calendars were found.')
        
    def testListCalendars(self):
        # Creates some rights (local roles) on self.ical :
        # user1: Read + Write
        # user2: Read
        # user3: none
        
        # current user is the portal manager, so it has Read + Write rights
        # user1 belongs to group1, so it has already both local roles
        # user3 hasn't got any right
        
        from Products.CMFCore.utils import getToolByName
        from AccessControl import getSecurityManager
        
        membershiptool=getToolByName(self.portal, 'portal_membership')
        membershiptool.setLocalRoles(self.ical, (self.user2.getUserName(),), self.localRoleReader, reindex=0)
        
        # Read tests
        self.failUnless(self.ical in self.ical.listCalendars(readable=1))
        self.failUnless(self.ical in self.ical.listCalendars(user='user1',readable=1))
        self.failUnless(self.ical in self.ical.listCalendars(user='user2',readable=1))
        self.failIf(self.ical in self.ical.listCalendars(user='user3',readable=1))
        
        # Write tests
        self.failUnless(self.ical in self.ical.listCalendars(writeable=1))
        self.failUnless(self.ical in self.ical.listCalendars(user='user1',writeable=1))
        self.failIf(self.ical in self.ical.listCalendars(user='user2',writeable=1))
        self.failIf(self.ical in self.ical.listCalendars(user='user3',writeable=1))
        
        # Combined tests
        self.failUnless(self.ical in self.ical.listCalendars(readable=1,writeable=1))
        self.failUnless(self.ical in self.ical.listCalendars(user='user1',readable=1,writeable=1))
        self.failIf(self.ical in self.ical.listCalendars(user='user2',readable=1,writeable=1))
        self.failIf(self.ical in self.ical.listCalendars(user='user3',readable=1,writeable=1))
        
        self.failUnless(self.ical.listAllCalendars()==self.ical.listCalendars())
        
        # Test with a user with no rights
        self.logout()
        self.login('user3')
        self.assertEqual(self.membership.getAuthenticatedMember().getUserName(), 'user3')
        self.failIf(self.ical in self.ical.listCalendars(readable=1))
        self.failIf(self.ical in self.ical.listCalendars(writeable=1))


if __name__ == '__main__':
    framework()
else:
    def test_suite():
        from unittest import TestSuite, makeSuite
        suite = TestSuite()
        suite.addTest(makeSuite(TestPloneiCalendarSecurity))
        return suite


