#
# PloneiCalendar tests
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.PloneTestCase import PloneTestCase

PloneTestCase.installProduct('PloneiCalendar')

PloneTestCase.setupPloneSite()

from Products.PloneiCalendar.global_symbols import *
from Acquisition import aq_base

try:
    import Log

    Log.LOG_LEVEL = Log.LOG_DEBUG

    Log.LOG_PROCESSOR = {
        Log.LOG_NONE: Log.logFile,
        Log.LOG_CRITICAL: Log.logFile,
        Log.LOG_ERROR: Log.logFile,
        Log.LOG_WARNING: Log.logFile,
        Log.LOG_NOTICE: Log.logFile,
        Log.LOG_DEBUG: Log.logFile,
        }
    from Log import *
    Log(LOG_NOTICE, "Starting %s at %d debug level" % (os.path.dirname(__file__), LOG_LEVEL, ))
except:
    print "Log module not available"
    LOG_DEBUG = None
    LOG_NOTICE = None
    LOG_WARNING = None
    LOG_ERROR = None
    LOG_CRITICAL = None
    def Log(*args, **kw):
        pass
    raise

class TestPloneiCalendarWorkflow(PloneTestCase.PloneTestCase):

    def installPloneiCalendar(self):
        self.qi = self.portal.portal_quickinstaller
        self.qi.installProduct('PloneiCalendar')
        
    def afterSetUp(self, ):
        self.installPloneiCalendar()
        
        self.catalog = self.portal.portal_catalog
        self.workflow = self.portal.portal_workflow
        self.membership = self.portal.portal_membership
        self.contentId = 'agenda'
        self.folder.invokeFactory(type_name='iCalendar', id=self.contentId)
        self.failUnless(self.catalog(id=self.contentId))
        self.ical = getattr(self.folder, self.contentId)

    #workflow
    def testWorkflowExists(self):
        """ test if workflow is installed """
        self.failUnless('ploneicalendar_workflow' in self.workflow.getWorkflowIds())
    
    def testPloneiCalendarWorkflow(self):
        """ test if the right workflow is assigned to PloneiCalendar product """
        wfs_lists = self.workflow.getWorkflowsFor(self.ical)
        self.assertEqual(len(wfs_lists), 1)
        ical_wf = wfs_lists[0]
        self.assertEqual(ical_wf.getId(), 'ploneicalendar_workflow')
    
    def testPloneiCalendarReviewState(self):
        """ test the default state of PloneiCalendar (private) """
        review_state = self.workflow.getInfoFor(self.ical, 'review_state')
        self.assertEqual(review_state, 'private')
    
    def testPublishPloneiCalendar(self):
        """ publish the PloneiCalendar """
        self.setRoles(['Owner'])
        self.workflow.doActionFor(self.ical, 'publish')
        self.assertEqual(self.workflow.getInfoFor(self.ical, 'review_state'), 'published')
        self.failUnless(self.catalog(id=self.contentId, review_state='published'))
        
    def testRetractPloneiCalendar(self):
        """ publish then retract the PloneiCalendar """
        self.setRoles(['Owner'])
        self.workflow.doActionFor(self.ical, 'publish')
        self.workflow.doActionFor(self.ical, 'retract')
        self.assertEqual(self.workflow.getInfoFor(self.ical, 'review_state'), 'private')
        self.failUnless(self.catalog(id=self.contentId, review_state='private'))


if __name__ == '__main__':
    framework()
else:
    def test_suite():
        from unittest import TestSuite, makeSuite
        suite = TestSuite()
        suite.addTest(makeSuite(TestPloneiCalendarWorkflow))
        return suite
